#ifndef LINT
static char *rcsid="$Id: clamav.c,v 1.10 2004/01/29 18:00:59 crosser Exp $";
#endif

/*
	$Log: clamav.c,v $
	Revision 1.10  2004/01/29 18:00:59  crosser
	typo in Id
	
	Revision 1.9  2004/01/29 17:32:47  crosser
	make info message, bump version
	
	Revision 1.8  2003/09/15 13:41:54  crosser
	report by setting variable
	
	Revision 1.7  2003/09/14 22:11:51  crosser
	removed dprint
	
	Revision 1.6  2003/09/10 21:24:24  crosser
	cosmetic
	
	Revision 1.5  2003/09/09 20:04:29  crosser
	stage name changed from "body" to "content"
	
	Revision 1.4  2003/09/08 13:01:57  crosser
	change ZMSCAN_REJECT to ZMSCAN_STOP
	
	Revision 1.3  2003/09/06 21:47:56  crosser
	don't scan text type
	
	Revision 1.2  2003/09/06 21:26:52  crosser
	Cool!  scanning works!
	
	Revision 1.1.1.1  2003/09/06 20:50:37  crosser
	Importe source
	
*/
                                                                                
/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#include "config.h"

#include <sys/types.h>
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef STDC_HEADERS
# include <stdio.h>
# include <string.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#include <zmscanner.h>
#include <clamav.h>

#include "report.h"

#define min(x,y) ((x<y)?x:y)

struct _cfg {
	int virnum;
	struct cl_node *root;
};

static int
clamav_setup(void **priv)
{
	struct _cfg *cfg;
	int rc;
	struct cl_node *root=NULL;
	int virnum=0;

	DPRINT(("clamav_setup starting\n"));

	rc=cl_loaddbdir(cl_retdbdir(),&root,&virnum);
	if (rc) {
		ERRLOG((LOG_ERR,"clamav: loaddbdir error: %s",
					cl_perror(rc)));
		return 1;
	}
	cl_buildtrie(root);

	cfg=(void *)malloc(sizeof(struct _cfg));
	cfg->virnum=virnum;
	cfg->root=root;
	(*priv)=(void *)cfg;

	ERRLOG((LOG_INFO,"clamav: loaded %d viruses signatures",virnum));
	ERRLOG((LOG_INFO,"clamav: %s (%s) OK",VERSION,rcsid));

	return 0;
}

static void
clamav_term(void **priv)
{
	struct _cfg *cfg=*((struct _cfg **)priv);

	if (cfg) {
		cl_freetrie(cfg->root);
		free(cfg);
	}
	*priv=NULL;
}

static int
clamav_scan(char *stage,int depth,slab_t data,varpool_t vp,void *priv)
{
	struct _cfg *cfg=(struct _cfg *)priv;
	int rc;
	char *virname;
	mattr_t *atr=data.atr;
	char ans[128];

	DPRINT(("clamav_scan enter\n"));

	if (!((atr) && (slab_size(atr->content_type)) &&
	      (strncasecmp(atr->content_type.beg,"text",
				min(4,slab_size(atr->content_type))) != 0))) {
		DPRINT(("not scanning text type\n"));
		return ZMSCAN_CONTINUE;
	}

	rc=cl_scanbuff(data.beg,slab_size(data),&virname,cfg->root);
	switch (rc) {
	case 0:
		DPRINT(("scan result clean\n"));
		return ZMSCAN_CONTINUE;
	case 1:
		ERRLOG((LOG_INFO,"clamav: blocked %s",virname));
		snprintf(ans,sizeof(ans),
			"-1 550 5.7.0 Found virus %s in the message",virname);
		vp_set_str(vp,VP_ANSWER_STR,ans);
		return ZMSCAN_STOP;
	default:
		ERRLOG((LOG_ERR,"clamav_scan result rc=%d\n",rc));
		return ZMSCAN_CONTINUE;
	}
}

ZMS_MODULE("content","clamav",clamav_setup,clamav_term,clamav_scan);

